// BLogWidget.cpp (this is -*- C++ -*-)
// 
// \author: Bjoern Giesler <bjoern@giesler.de>
// 
// 
// 
// $Author: giesler $
// $Locker$
// $Revision$
// $Date: 2002-08-19 10:41:28 +0200 (Mon, 19 Aug 2002) $

/* system includes */
#include <nds/jtypes.h>

/* my includes */
#include "BLogWidget.h"
#include "BTheme.h"

BLogWidget::BLogWidget(const BRect& frame, BFont* font):
  BWidget(frame)
{
  this->font = font;

  maxlines = 100;
  firstline = 0;
  autoscroll = true;
  setTextDirection(BFont::DIR_LEFTTORIGHT);
}

void
BLogWidget::draw(BImage& img)
{
  BWidget::draw(img);
  BRect clip = BTheme::currentTheme()->contentRectForWidget(*this);
  clip = clip.insetRect(1, 1, 0, 0);

  if(firstline != 0 || !atEnd())
    clip.sz.width -= 7;

  unsigned int fh = font->height();

  BPoint point;
  switch(dir)
    {
    case BFont::DIR_LEFTTORIGHT:
      point = clip.pt;
      break;

    case BFont::DIR_TOPTOBOTTOM:
      point = BPoint(clip.pt.x+clip.sz.width, clip.pt.y+1);
      break;

    case BFont::DIR_RIGHTTOLEFT:
      point = BPoint(clip.pt.x+clip.sz.width-1, clip.pt.y+clip.sz.height-1);
      break;

    case BFont::DIR_BOTTOMTOTOP:
      point = BPoint(clip.pt.x, clip.pt.y+clip.sz.height-1);
      break;
    }

  for(unsigned int i=firstline;
      i<lines.size() && i < firstline + numlines;
      i++)
    {
      uint16 color = RGB15(0, 0, 0);
      switch(lines[i].level)
	{
	case LOG_FATAL:
	case LOG_CRIT:
	case LOG_ERROR:
	  color = RGB15(20, 0, 0);
	  break;
	case LOG_WARN:
	  color = RGB15(15, 15, 0);
	  break;
	case LOG_INFO:
	  color = RGB15(0, 0, 0);
	  break;
	case LOG_BLAH:
	  color = RGB15(8, 8, 8);
	  break;
	}
      
      font->drawString(img, point,
		       lines[i].text, color, clip,
		       dir);
      switch(dir)
	{
	case BFont::DIR_LEFTTORIGHT:
	  point.y += fh;
	  break;
	  
	case BFont::DIR_RIGHTTOLEFT:
	  point.y -= fh;
	  break;

	case BFont::DIR_TOPTOBOTTOM:
	  point.x -= fh;
	  break;

	case BFont::DIR_BOTTOMTOTOP:
	  point.x += fh;
	  break;
	}
    }

  if(firstline != 0 || !atEnd())
    {
      clip.sz.width += 7;
      uint16* bytes = (uint16*)img.bytes();
      
      BPoint p(clip.pt.x + clip.sz.width - 6,
	       clip.pt.y);
      for(unsigned int y=clip.pt.y;
	  y<clip.pt.y + clip.sz.height + 1; y++)
	bytes[y*img.width() + p.x] = 1 << 15;

      float thumbheight = ((float)numlines / (float)lines.size());
      float thumbpos = ((float)firstline / (float)(lines.size() - numlines)) *
	(1.0f - thumbheight);

      int tp = (int)((clip.sz.height - 12)*thumbpos);
      int th = (int)((clip.sz.height - 12)*thumbheight);
      
      BRect thumb(clip.pt.x + clip.sz.width - 4,
		  clip.pt.y + tp + 6,
		  5, th+3);

      thumb.fill(img, RGB15(0, 0, 0) | 1<<15);
      
      p = BPoint(clip.pt.x + clip.sz.width - 4,
		 clip.pt.y + 4);
      
      bytes[p.y*img.width() + p.x] = 1 << 15;
      bytes[p.y*img.width() + p.x + 1] = 1 << 15;
      bytes[p.y*img.width() + p.x + 2] = BIT(15);
      bytes[p.y*img.width() + p.x + 3] = BIT(15);
      bytes[p.y*img.width() + p.x + 4] = BIT(15);
      bytes[(p.y-1)*img.width() + p.x + 1] = BIT(15);
      bytes[(p.y-1)*img.width() + p.x + 2] = BIT(15);
      bytes[(p.y-1)*img.width() + p.x + 3] = BIT(15);
      bytes[(p.y-2)*img.width() + p.x + 2] = BIT(15);

      p = BPoint(clip.pt.x + clip.sz.width - 4,
		 clip.pt.y + clip.sz.height - 3);
      
      bytes[p.y*img.width() + p.x] = 1 << 15;
      bytes[p.y*img.width() + p.x + 1] = 1 << 15;
      bytes[p.y*img.width() + p.x + 2] = BIT(15);
      bytes[p.y*img.width() + p.x + 3] = BIT(15);
      bytes[p.y*img.width() + p.x + 4] = BIT(15);
      bytes[(p.y+1)*img.width() + p.x + 1] = BIT(15);
      bytes[(p.y+1)*img.width() + p.x + 2] = BIT(15);
      bytes[(p.y+1)*img.width() + p.x + 3] = BIT(15);
      bytes[(p.y+2)*img.width() + p.x + 2] = BIT(15);
    }
}

void
BLogWidget::log(LogLevel level, const std::string& text)
{
  LogEntry entry = {level, text};
  lines.push_back(entry);
  while(lines.size() > maxlines) lines.pop_front();

  if(autoscroll)
    {
      if(lines.size() <= numlines)
	firstline = 0;
      else
	firstline = lines.size() - numlines;
    }
}

void
BLogWidget::setMaxLines(unsigned int maxlines)
{
  if(numlines > maxlines)
    this->maxlines = numlines;
  else
    this->maxlines = maxlines;
}

void
BLogWidget::setFirstLine(unsigned int line)
{
  if(line > lines.size() - numlines ||
     lines.size() <= numlines) return;
  firstline = line;
}

bool
BLogWidget::atEnd()
{
  if(lines.size() < numlines ||
     firstline == lines.size() - numlines)
    return true;
  return false;
}

void
BLogWidget::setAutoscroll(bool autoscroll)
{
  this->autoscroll = autoscroll;
}

void
BLogWidget::setTextDirection(BFont::TextDirection dir)
{
  this->dir = dir;
  
  BRect clip = BTheme::currentTheme()->contentRectForWidget(*this);

  switch(dir)
    {
    case BFont::DIR_LEFTTORIGHT:
    case BFont::DIR_RIGHTTOLEFT:
      numlines = clip.sz.height/font->height();
      break;
    case BFont::DIR_TOPTOBOTTOM:
    case BFont::DIR_BOTTOMTOTOP:
      numlines = clip.sz.width/font->height();
      break;
    }
}
